<?php
/*
Plugin Name: En Garde
Plugin URI: http://blog.tremlas.com/en-garde-plugin-details/
Description: Adds En Garde shortcodes
Version: 2.0.0.20171013-1012
Author: David Vines
Author URI: http://tremlas.com
License:  Creative Commons License
*/
/* © Copyright 2014, 2017 David Vines

   Creative Commons License

   This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
*/

// Debugging function in case we need it
if ( !function_exists('djv_write_log') ) {
	function djv_write_log ( $log )  {
		if ( true === WP_DEBUG ) {
			if ( is_array( $log ) || is_object( $log ) ) {
				error_log( print_r( $log, true ) );
			} else {
				error_log( $log );
			}
		}
	}
}

// Initial month
add_option( 'djv_engarde_options', array(
		'month' => 'février 1924',
        'femmes' => 'Auriane Delaunay,Flora Pernet',
        'email' => 'engarde@blog.tremlas.com'
	)
);


// An admin page to allow the month to be modified
class DjvengardeSettingsPage
{
	private $options;
	
	public function __construct() {
		add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'page_init' ) );
	}
	
	public function add_plugin_page() {
		add_options_page(
			'En Garde Admin',
			'En Garde',
			'manage_options',
			'djv-engarde-setting-admin',
			array( $this, 'create_admin_page' ) 
		);
	}
	
	public function create_admin_page() {
		$this->options = get_option('djv_engarde_options');
		?>
		<div class="wrap">
			<?php screen_icon(); ?>
			<h2>En Garde Options</h2>
			<form method="post" action="options.php">
			<?php
				settings_fields('djv_engarde_option_group');
				do_settings_sections('djv-engarde-setting-admin');
				submit_button(); 
			?>
			</form>
		</div>
		<?php
	}
	
	public function page_init() {
		register_setting(
			'djv_engarde_option_group',
			'djv_engarde_options',
			array( $this, 'sanitize' )
		);
		add_settings_section(
			'djv_engarde_settings_section_id',
			'En Garde Settings',
			array( $this, 'print_section_info' ),
			'djv-engarde-setting-admin'
		);
		add_settings_field(
			'month',
			'The month for which orders are expected',
			array( $this, 'month_callback' ),
			'djv-engarde-setting-admin',
			'djv_engarde_settings_section_id'
		);
		add_settings_field(
			'femmes',
			'Las femmes fatales',
			array( $this, 'femmes_callback' ),
			'djv-engarde-setting-admin',
			'djv_engarde_settings_section_id'
		);
		add_settings_field(
			'email',
			'Email to which to submit orders',
			array( $this, 'email_callback' ),
			'djv-engarde-setting-admin',
			'djv_engarde_settings_section_id'
		);
	}
	
	public function sanitize( $input ) {
		$new_input = array();
		if ( isset( $input['month'] ) ) {
			$new_input['month'] = $input['month'];
		}
		if ( isset( $input['femmes'] ) ) {
			$new_input['femmes'] = $input['femmes'];
		}
		if ( isset( $input['email'] ) && (filter_var($input['email'], FILTER_VALIDATE_EMAIL) ) ) {
			$new_input['email'] = $input['email'];
		}
		return $new_input;
	}
	
	public function print_section_info() {
		print "Enter the settings for the En Garde plugin.";
	}
	
    public function month_callback() {
		$settings = array(
			'textarea_name' => 'djv_engarde_options[month]',
			'teeny' => true,
			'textarea_rows' => 1,
			'tabindex' => 1,
			'media_buttons' => false
		);
		wp_editor(
			isset( $this->options['month'] ) ? $this->options['month'] : '',
			'month',
			$settings
		);
	}
	
    public function email_callback() {
		$settings = array(
			'textarea_name' => 'djv_engarde_options[email]',
			'teeny' => true,
			'textarea_rows' => 1,
			'tabindex' => 1,
			'media_buttons' => false
		);
		wp_editor(
			isset( $this->options['email'] ) ? $this->options['email'] : '',
			'email',
			$settings
		);
	}
	
	public function femmes_callback() {
		$settings = array(
			'textarea_name' => 'djv_engarde_options[femmes]',
			'teeny' => true,
			'textarea_rows' => 5,
			'tabindex' => 1,
			'media_buttons' => false
		);
		wp_editor(
			isset( $this->options['femmes'] ) ? $this->options['femmes'] : '',
			'femmes',
			$settings
		);
	}
	
}
if (is_admin()) {
	$djv_engarde_settings_page = new DjvengardeSettingsPage();
}

// Also provide a link on the plugins listings page
function djv_add_engarde_plugin_action_links( $links ) {
	return array_merge(
			array( 'settings' => '<a href="' . get_bloginfo( 'wpurl' ) . '/wp-admin/options-general.php?page=djv-engarde-setting-admin">Settings</a>'
			),
			$links
		);
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'djv_add_engarde_plugin_action_links' );

/** 
 * Shortcode to add the version of the plugin
 */
function djv_engarde_engardeVersion_shortcode() {
	return 'En Garde Version 2.0.0.20171013-1012';
} 
add_shortcode( 'enGardeVersion', 'djv_engarde_engardeVersion_shortcode' );

function djv_engarde_engardeCharacter_attr_shortcode_general( $name, $attr ) {
 if ( isset($attr) && isset( $attr['is'] ) ) {
  $cur = $attr['is'];
 } else if ( isset($attr) && isset( $attr['has'] ) ) {
  $cur = $attr['has'];
 } else {
  $cur = 'unknown';
 }
 $notes = '';
 if ( isset($attr) && isset( $attr['initial'] ) ) {
  $notes .= 'Started at ' . $attr['initial'];
 }
 if ( isset($attr) && isset( $attr['practice'] ) ) {
  if ( $notes !== '' ) { $notes .= '. '; }
  $notes .= $attr['practice'] . ' of practice';
 } else if ( isset($attr) && isset( $attr['notes'] ) ) {
  if ( $notes !== '' ) { $notes .= '. '; }
  $notes .= $attr['notes'];
 }
 return '<tr><td>' . $name . '</td><td>' . $cur . '</td><td>'  . $notes . '</td></tr>';
}


function djv_engarde_engardeCharacter_str_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Strength', $attr );
}

function djv_engarde_engardeCharacter_rx_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Rapier Expertise', $attr );
}

function djv_engarde_engardeCharacter_sx_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Sabre Expertise', $attr );
}

function djv_engarde_engardeCharacter_cx_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Cutlass Expertise', $attr );
}

function djv_engarde_engardeCharacter_con_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Constitution', $attr );
}

function djv_engarde_engardeCharacter_end_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Endurance', $attr );
}

function djv_engarde_engardeCharacter_ma_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Military Ability', $attr );
}

function djv_engarde_engardeCharacter_cash_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Cash', $attr );
}

function djv_engarde_engardeCharacter_allow_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Allowance, Salary &amp; Pension', $attr );
}

function djv_engarde_engardeCharacter_club_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Club', $attr );
}

function djv_engarde_engardeCharacter_reg_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Regiment &amp; Rank', $attr );
}

function djv_engarde_engardeCharacter_app_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Appointment', $attr );
}

function djv_engarde_engardeCharacter_mistress_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Female Companion', $attr );
}

function djv_engarde_engardeCharacter_mentions_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Mentions in Dispatches', $attr );
}

function djv_engarde_engardeCharacter_stable_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Stable', $attr );
}

function djv_engarde_engardeCharacter_level_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Social Level', $attr );
}

function djv_engarde_engardeCharacter_location_shortcode( $attr ) {
 return djv_engarde_engardeCharacter_attr_shortcode_general( 'Location', $attr );
}

function djv_engarde_engardeCharacter_shortcode( $attr, $content = null ) {
 if ( isset($attr) && isset( $attr['id'] ) ) {
  $id = $attr['id'];
 } else {
  $id = '';
 }
 
 $text = '' . PHP_EOL;
 $text .= '<table id="djv-engarde-stats-' . $id . '">' . PHP_EOL;
 $text .= '<tr><th>Attribute</th><th>Currently</th><th>Notes</th></tr>' . PHP_EOL;
 
 add_shortcode('str', 'djv_engarde_engardeCharacter_str_shortcode');
 add_shortcode('rapier', 'djv_engarde_engardeCharacter_rx_shortcode');
 add_shortcode('sabre', 'djv_engarde_engardeCharacter_sx_shortcode');
 add_shortcode('cutlass', 'djv_engarde_engardeCharacter_cx_shortcode');
 add_shortcode('con', 'djv_engarde_engardeCharacter_con_shortcode');
 add_shortcode('end', 'djv_engarde_engardeCharacter_end_shortcode');
 add_shortcode('ma', 'djv_engarde_engardeCharacter_ma_shortcode');
 add_shortcode('cash', 'djv_engarde_engardeCharacter_cash_shortcode');
 add_shortcode('allowance', 'djv_engarde_engardeCharacter_allow_shortcode');
 add_shortcode('club', 'djv_engarde_engardeCharacter_club_shortcode');
 add_shortcode('regiment', 'djv_engarde_engardeCharacter_reg_shortcode');
 add_shortcode('appointment', 'djv_engarde_engardeCharacter_app_shortcode');
 add_shortcode('mistress', 'djv_engarde_engardeCharacter_mistress_shortcode');
 add_shortcode('mentions', 'djv_engarde_engardeCharacter_mentions_shortcode');
 add_shortcode('stable', 'djv_engarde_engardeCharacter_stable_shortcode');
 add_shortcode('level', 'djv_engarde_engardeCharacter_level_shortcode');
 add_shortcode('location', 'djv_engarde_engardeCharacter_location_shortcode');
 
 $text .= do_shortcode( $content );
 $text = preg_replace( '!<br ?/>!', '', $text );
 
 remove_shortcode('str');
 remove_shortcode('rapier');
 remove_shortcode('sabre');
 remove_shortcode('cutlass');
 remove_shortcode('con');
 remove_shortcode('end');
 remove_shortcode('ma');
 remove_shortcode('cash');
 remove_shortcode('club');
 remove_shortcode('regiment');
 remove_shortcode('appointment');
 remove_shortcode('mistress');
 remove_shortcode('mentions');
 remove_shortcode('stable');
 remove_shortcode('level');
 remove_shortcode('location');
 
 $text .= '</table>' . PHP_EOL;
  
 return $text;
}
add_shortcode( 'enGardeCharacter', 'djv_engarde_engardeCharacter_shortcode' );

/**
 * Shortcode to add order template
 */
function djv_engarde_engardeOrderTemplate_shortcode( $attr, $content = null ) {
 $plugin_url = plugin_dir_url(__FILE__);
 wp_enqueue_script('djv_engarde_engarde',$plugin_url.'js/engarde/engarde.js');
 wp_enqueue_style('djv_engarde_enGarde_style',$plugin_url.'css/engarde.css');
 
 if ( isset($attr) && isset( $attr['id'] ) ) {
  $id = $attr['id'];
 } else {
  $id = 'unknown';
 }
 
 $prefix = 'djv-engarde-' . $id . '-';
 $text = '' . PHP_EOL;
 $text .= '<div id="' . $prefix . 'orders-div">' . PHP_EOL;
 $text .= '<hr>' . PHP_EOL;
 $text .= '<div class="arrow-right" id="' . $prefix . 'arrow" onClick="expandCollapseSection(\'' . $prefix . 'arrow\',\'djv-engarde-' . $id . '-order-form\');"></div>' . PHP_EOL;
 $text .= '<h2 class="form-title" onClick="expandCollapseSection(\'' . $prefix . 'arrow\',\'djv-engarde-' . $id . '-order-form\');">Order Template</h2><br>' . PHP_EOL;
 $text .= '<div id="djv-engarde-' . $id . '-order-form" style="display: none">' . PHP_EOL;
 $text .= '<p>You may find the <a href="http://www.engarde.co.uk/engarde!Tables.pdf">tables available on the En Garde</a> web site useful while completing these orders.</p>' . PHP_EOL;
 
 $text .= '<table>' . PHP_EOL;
 $text .= '<tr><th style="width: 12%;">Order Item</th><th style="width: 58%;">Choice</th><th style="width: 30%;">Effects</th></tr>' . PHP_EOL;
 
 $text .= '<tr><td>Character Name</td><td><span id="' . $prefix . 'name">' . esc_html( get_the_title() ) . '</span></td><td></td></tr>' . PHP_EOL;
 $text .= '<tr><td>Month</td><td><span id="' . $prefix . 'month">' . get_option('djv_engarde_options')['month'] . '</span></td><td></td></tr>' . PHP_EOL;
 
 $text .= '<tr><td>Support costs</td><td>' . PHP_EOL;
 $text .= '<input type="radio" name="' . $prefix . 'support" id="' . $prefix . 'support-none"/><span id="' . $prefix . 'support-none-label">None</span>';
 $text .= '<input type="radio" name="' . $prefix . 'support" id="' . $prefix . 'support-std" checked="checked"/><span id="' . $prefix . 'support-std-label">Standard</span>';
 $text .= '<input type="radio" name="' . $prefix . 'support" id="' . $prefix . 'support-con"/><span id="' . $prefix . 'support-con-label">Conspicious</span>';
 $text .= '</td><td id="' . $prefix . 'support-effect"></td>' . PHP_EOL;
 
 $text .= '<tr><td>Regiment</td><td>' . PHP_EOL;
 $text .= '<input type="checkbox" id="' . $prefix . 'regiment-leave"/><span id="' . $prefix . 'regiment-leave-label">Leave current regiment</span>';
 $text .= '<input type="checkbox" id="' . $prefix . 'regiment-join"/><span id="' . $prefix . 'regiment-join-label">Apply to join:</span>';
 $text .= '<select id="' . $prefix . 'regiment"><option selected></option>' . PHP_EOL;
 $text .= '<option>Dragoon Guards</option><option>Queen\'s Own Carabiniers</option>' . PHP_EOL;
 $text .= '<option>Royal Foot Guards</option><option>King\'s Musketeers</option><option>Cardinal\'s Guard</option>' . PHP_EOL;
 $text .= '<option>Crown Prince Cuirassiers</option><option>Archduke Leopold Cuirassiers</option>' . PHP_EOL;
 $text .= '<option>Grand Duke Maximillian Dragoons</option><option>Princess Louisa Light Dragoons</option>' . PHP_EOL;
 $text .= '<option>Royal Marines</option><option>Picardy Musketeers</option>' . PHP_EOL;
 $text .= '<option>13th Fusiliers</option><option>53rd Fusiliers</option>' . PHP_EOL;
 $text .= '<option>27th Musketeers</option><option>4th Arquebusiers</option>' . PHP_EOL;
 $text .= '<option>69th Arquebusiers</option><option>Gascon Regiment</option>' . PHP_EOL;
 $text .= '</select></td><td id="' . $prefix . 'regiment-effect"></td></tr>' . PHP_EOL;
 
 $text .= '<tr><td>Promotion</td><td>' . PHP_EOL;
 $text .= '<input type="checkbox" id="' . $prefix . 'promote"/><span id="' . $prefix . 'promote-label">Apply for promotion to:</span>';
 $text .= '<select id="' . $prefix . 'rank"><option selected></option>' . PHP_EOL;
 $text .= '<option>Subaltern</option><option>Captain</option><option>Major</option><option>Lieutenant Colonel</option><option>Colonel</option>' . PHP_EOL;
 $text .= '</select></td><td id="' . $prefix . 'rank-effect"></td></tr>' . PHP_EOL;
 
 $text .= '<tr><td>Club membership</td><td>' . PHP_EOL;
 $text .= '<input type="checkbox" id="' . $prefix . 'club-leave"/><span id="' . $prefix . 'club-leave-label">Leave current club</span>';
 $text .= '<input type="checkbox" id="' . $prefix . 'club-join"/><span id="' . $prefix . 'club-join-label">Join new club:</span>';
 $text .= '<select id="' . $prefix . 'club"><option selected></option>' . PHP_EOL;
 $text .= '<option>Bothwell\'s</option><option>Hunter\'s</option><option>The Horse Guards</option><option>The Blue Gables</option><option>The Frog &amp; Peach</option><option>Red Phillips</option>' . PHP_EOL;
 $text .= '</select></td><td id="' . $prefix . 'club-effect"></td></tr>' . PHP_EOL;
 
 $text .= '<tr><td>Location this month</td><td>' . PHP_EOL;
 $text .= '<input type="radio" name="' . $prefix . 'location" id="' . $prefix . 'location-paris" checked="checked"/><span id="' . $prefix . 'location-paris-label">In Paris</span>';
 $text .= '<input type="radio" name="' . $prefix . 'location" id="' . $prefix . 'location-at-war"/><span id="' . $prefix . 'location-at-war-label">Gone with Regiment</span>';
 $text .= '<input type="radio" name="' . $prefix . 'location" id="' . $prefix . 'location-frontier"/><span id="' . $prefix . 'location-frontier-label">With Frontier Regiment</span>';
 $text .= '<input type="radio" name="' . $prefix . 'location" id="' . $prefix . 'location-other"/><span id="' . $prefix . 'location-other-label">Other</span>';
 $text .= '<br><p>Notes:</p><textarea id="' . $prefix . 'location-notes" rows="3" class="dataentry" placeholder="Add additional notes..."></textarea>' . PHP_EOL;
 $text .= '</td><td id="' . $prefix . 'location-effect"></td>' . PHP_EOL;
 
 $femmes = get_option('djv_engarde_options')['femmes'];
 $femmes = str_replace(array("\r", "\n"), '', $femmes); // To remove any line breaks introduced for readability on the admin panel :)
 $femmes = explode(';',$femmes);
 sort($femmes);
 for( $i = 1; $i < 5; $i++) {
  $text .= '<tr id="' . $prefix . 'week-' . $i . '-row"><td><h2 id="' . $prefix . 'week-' . $i . '-heading">Week ' . $i . '</h2></td><td>' . PHP_EOL;
  $text .= '<select id="' . $prefix . 'week-' . $i . '"><option selected>On Duty</option>' . PHP_EOL;
  $text .= '<option>Visit Club</option><option>Visit Bawdyhouse</option>' . PHP_EOL;
  $text .= '<option>Court new female companion</option>' . PHP_EOL;
  $text .= '<option>Visit female companion</option>' . PHP_EOL;
  $text .= '<option>Practice Fencing</option>' . PHP_EOL; 
  $text .= '<option>Other</option>' . PHP_EOL; 
  $text .= '</select>' . PHP_EOL;
  $text .= '<div id="' . $prefix . 'week-' . $i . '-club-details" style="display: none;"><br>' . PHP_EOL;
  $text .= '<input type="checkbox" id="' . $prefix . 'week-' . $i . '-club-details-carouse"/>Carouse';
  $text .= '<input type="checkbox" id="' . $prefix . 'week-' . $i . '-club-details-companion"/>With Companion';
  $text .= '<input type="checkbox" id="' . $prefix . 'week-' . $i . '-club-details-gamble"/>Gamble';
  $text .= '</div><div id="' . $prefix . 'week-' . $i . '-bawdy-details" style="display: none;"><br>';
  $text .= '<input type="checkbox" id="' . $prefix . 'week-' . $i . '-bawdy-details-carouse" checked="checked" disabled="disabled"/>Carouse';
  $text .= '<input type="checkbox" id="' . $prefix . 'week-' . $i . '-bawdy-details-companion"/>With Bawdyhouse Companion';
  $text .= '<input type="checkbox" id="' . $prefix . 'week-' . $i . '-bawdy-details-gamble"/>Gamble';
  $text .= '</div><div id="' . $prefix . 'week-' . $i . '-gamble-details" style="display: none;"><br>' . PHP_EOL;
  $text .= '<input type="number" class="onedigitnumber" id="' . $prefix . 'week-' . $i . '-gamble-details-rounds" min="1" max="9" step="1" value="1">';
  $text .= ' <span id="' . $prefix . 'week-' . $i . '-gamble-details-rounds-label">round</span>';
  $text .= ' gambling <input type="number" class="fivedigitnumber" id="' . $prefix . 'week-' . $i . '-gamble-details-bet" min="1" max="10000" value="1">';
  $text .= ' <span id="' . $prefix . 'week-' . $i . '-gamble-details-bet-label">Crown</span>, cutting on a house roll of ';
  $text .= '<input type="number" class="onedigitnumber" id="' . $prefix . 'week-' . $i . '-gamble-details-cut" min="1" max="7" step="1" value="5">';
  $text .= '</div><div id="' . $prefix . 'week-' . $i . '-court-details" style="display:none;"><br>' . PHP_EOL;
  $text .= '<select id="' . $prefix . 'week-' . $i . '-court-companion">' . PHP_EOL;
  foreach($femmes as $femme) {
   $text .= '<option>' . $femme . '</option>' . PHP_EOL;
  }
  $text .= '</select>' . PHP_EOL;
  $text .= '</div><div id="' . $prefix . 'week-' . $i . '-practice-details" style="display: none;"><br>' . PHP_EOL;
  $text .= '<input type="radio" name="' . $prefix . 'week-' . $i . '-practice-choice" id="' . $prefix . 'week-' . $i . '-practice-rapier" checked="checked"/>Rapier';
  $text .= '<input type="radio" name="' . $prefix . 'week-' . $i . '-practice-choice" id="' . $prefix . 'week-' . $i . '-practice-sabre"/>Sabre';
  $text .= '<input type="radio" name="' . $prefix . 'week-' . $i . '-practice-choice" id="' . $prefix . 'week-' . $i . '-practice-cutlass"/>Cutlass';
  $text .= '</div><br><p><span id="' . $prefix . 'week-' . $i . '-notes-label">Notes:</p><textarea id="' . $prefix . 'week-' . $i . '-notes" rows="3" class="dataentry" placeholder="Add additional notes..."></textarea>' . PHP_EOL;
  $text .= '</td><td id="' . $prefix . 'week-' . $i . '-effect"></td></tr>' . PHP_EOL;
 }

 $text .= '<tr><td><h2>Other</h2></td><td>' . PHP_EOL;
 $text .= '<p>Notes:</p><textarea id="' . $prefix . 'other-notes" rows="3" class="dataentry" placeholder="Add additional notes..."></textarea>' . PHP_EOL;
 $text .= '</td><td id="' . $prefix . 'other-notes-effect"></td>' . PHP_EOL;
 
 $text .= '<tr><td><h2>Submit orders</h2></td>' . PHP_EOL;
 $text .= '<td><form action="' . esc_url( admin_url('admin-post.php')) . '" method="post"/>' . PHP_EOL;
 $text .= '<p><label for="' . $prefix . 'passcode">Passcode for ' . esc_html( get_the_title() ) . ':</label></p>' . PHP_EOL;
 $text .= '<p><input type="password" name="passcode" id="' . $prefix . 'passcode" required="required"/></p>' . PHP_EOL;
 $text .= '<p><label for="' . $prefix . 'ccemail">Email Address to which the orders will be copied</label></p>' . PHP_EOL;
 $text .= '<p><input type="email" name="ccemail" id="' . $prefix . 'ccemail"/></p>' . PHP_EOL;
 $text .= wp_nonce_field('submit_orders', 'nonce', true, false) . PHP_EOL;
 $text .= '<input type="hidden" name="postid" value="' . get_the_ID() . '"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="characterName" value="' . esc_html( get_the_title() ) . '"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="month" value="' . get_option('djv_engarde_options')['month'] . '"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="support" value="" id="' . $prefix . 'form-support"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="support-effect" value="" id="' . $prefix . 'form-support-effect"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="regiment" value="" id="' . $prefix . 'form-regiment"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="regiment-effect" value="" id="' . $prefix . 'form-regiment-effect"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="rank" value="" id="' . $prefix . 'form-rank"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="rank-effect" value="" id="' . $prefix . 'form-rank-effect"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="club" value="" id="' . $prefix . 'form-club"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="club-effect" value="" id="' . $prefix . 'form-club-effect"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="location" value="" id="' . $prefix . 'form-location"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="location-notes" value="" id="' . $prefix . 'form-location-notes"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="location-effect" value="" id="' . $prefix . 'form-location-effect"/>' . PHP_EOL;
 for( $i = 1; $i < 5; $i++) {
  $text .= '<input type="hidden" name="week' . $i . '" value="" id="' . $prefix . 'form-week' . $i .'"/>' . PHP_EOL;
  $text .= '<input type="hidden" name="week' . $i . '-notes" value="" id="' . $prefix . 'form-week' . $i .'-notes"/>' . PHP_EOL;
  $text .= '<input type="hidden" name="week' . $i . '-effect" value="" id="' . $prefix . 'form-week' . $i .'-effect"/>' . PHP_EOL;
 }
 $text .= '<input type="hidden" name="other-notes" value="" id="' . $prefix . 'form-other-notes"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="other-effect" value="" id="' . $prefix . 'form-other-effect"/>' . PHP_EOL;
 $text .= '<input type="hidden" name="action" value="djv_engarde_submit_orders"/>' . PHP_EOL;
 $text .= '<p><input type="submit" value="Submit orders for ' . esc_html( get_the_title() ) . '" id="' . $prefix . 'submit"/>' . PHP_EOL;
 $text .= '</form></td><td><p>Passcode is required. If you omit the email address your orders will be copied to the last known email address used for this character (invalid email address will fail and cause the last known email address to be deleted).</p></td></tr>' . PHP_EOL; 
 
 $text .= '</table>' . PHP_EOL;
 
 $text .= '</div>' . PHP_EOL;
 $text .= '<hr>' . PHP_EOL;
 $text .= '</div>' . PHP_EOL;
  return $text;
}
add_shortcode( 'enGardeOrderTemplate', 'djv_engarde_engardeOrderTemplate_shortcode' );

function djv_engarde_verify_passcode( $postid, $name, $passcode ) {
 if ( empty( $postid ) ) {
  return false;
 }
 $post = get_post( $postid );
 if ( esc_html( $post->post_title ) !== esc_html( stripslashes( $name ) ) ) {
  return false;
 }
 return ( $passcode == get_post_meta( $postid, 'passcode', true ) );
}

function djv_engarde_submit_orders() {
 status_header(200);
 if ( isset( $_REQUEST['nonce'] ) ) {
  wp_verify_nonce('submit_orders', 'nonce');
  if ( isset( $_REQUEST['passcode'] ) && isset( $_REQUEST['postid'] ) && isset( $_REQUEST['characterName'] ) && djv_engarde_verify_passcode( $_REQUEST['postid'], $_REQUEST['characterName'], $_REQUEST['passcode']) ) {
   if ( isset( $_REQUEST['month'] ) && 
        isset( $_REQUEST['support'] ) && 
        isset( $_REQUEST['support-effect'] ) &&
        isset( $_REQUEST['regiment'] ) &&
        isset( $_REQUEST['regiment-effect'] ) &&
        isset( $_REQUEST['rank'] ) &&
        isset( $_REQUEST['rank-effect'] ) &&
        isset( $_REQUEST['club'] ) &&
        isset( $_REQUEST['club-effect'] ) &&
        isset( $_REQUEST['location'] ) &&
        isset( $_REQUEST['location-notes'] ) &&
        isset( $_REQUEST['location-effect'] ) &&
        isset( $_REQUEST['week1'] ) &&
        isset( $_REQUEST['week1-notes'] ) &&
        isset( $_REQUEST['week1-effect'] ) &&
        isset( $_REQUEST['week2'] ) &&
        isset( $_REQUEST['week2-notes'] ) &&
        isset( $_REQUEST['week2-effect'] ) &&
        isset( $_REQUEST['week3'] ) &&
        isset( $_REQUEST['week3-notes'] ) &&
        isset( $_REQUEST['week3-effect'] ) &&
        isset( $_REQUEST['week4'] ) &&
        isset( $_REQUEST['week4-notes'] ) &&
        isset( $_REQUEST['week4-effect'] ) &&
        isset( $_REQUEST['other-notes'] ) &&
        isset( $_REQUEST['other-effect'] ) ) {
     $text .= '<table>' . PHP_EOL;
     $text .= '<tr><td>Name</td><td>' . esc_html( stripslashes( $_REQUEST['characterName'] ) ) . '</td><td></td></tr>' . PHP_EOL;
     $text .= '<tr><td>Month</td><td>' . esc_html( stripslashes( $_REQUEST['month'] ) ) . '</td><td></td></tr>' . PHP_EOL;
     $text .= '<tr><td>Support</td><td>' . esc_html( stripslashes( $_REQUEST['support'] ) ) . '</td><td>' . esc_html( stripslashes( $_REQUEST['support-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Regiment</td><td>' . esc_html( stripslashes( $_REQUEST['regiment'] ) ) . '</td><td>' . esc_html( stripslashes( $_REQUEST['regiment-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Rank</td><td>' . esc_html( stripslashes( $_REQUEST['rank'] ) ) . '</td><td>' . esc_html( stripslashes( $_REQUEST['rank-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Club</td><td>' . esc_html( stripslashes( $_REQUEST['club'] ) ) . '</td><td>' . esc_html( stripslashes( $_REQUEST['club-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Location</td><td>' . esc_html( stripslashes( $_REQUEST['location'] ) ) . '<p>' . esc_html( stripslashes( $_REQUEST['location-notes'] ) ) . '</p></td><td>' . esc_html( stripslashes( $_REQUEST['location-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Week 1</td><td>' . esc_html( stripslashes( $_REQUEST['week1'] ) ) . '<p>' . esc_html( stripslashes( $_REQUEST['week1-notes'] ) ) . '</p></td><td>' . esc_html( stripslashes( $_REQUEST['week1-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Week 2</td><td>' . esc_html( stripslashes( $_REQUEST['week2'] ) ) . '<p>' . esc_html( stripslashes( $_REQUEST['week2-notes'] ) ) . '</p></td><td>' . esc_html( stripslashes( $_REQUEST['week2-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Week 3</td><td>' . esc_html( stripslashes( $_REQUEST['week3'] ) ) . '<p>' . esc_html( stripslashes( $_REQUEST['week3-notes'] ) ) . '</p></td><td>' . esc_html( stripslashes( $_REQUEST['week3-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Week 4</td><td>' . esc_html( stripslashes( $_REQUEST['week4'] ) ) . '<p>' . esc_html( stripslashes( $_REQUEST['week4-notes'] ) ) . '</p></td><td>' . esc_html( stripslashes( $_REQUEST['week4-effect'] ) ) . '</td></tr>' . PHP_EOL;
     $text .= '<tr><td>Other Notes</td><td><p>' . esc_html( stripslashes( $_REQUEST['other-notes'] ) ). '</p></td><td>' . esc_html( stripslashes( $_REQUEST['other-effect'] ) ). '</td></tr>';
     $text .= '</table>';
     $text .= '<p>Orders recorded by En Garde Software Version 2.0.0.20171013-1012</p>';
     $to = get_option('djv_engarde_options')['email'];
     $cc = "";
     if (!empty( $_REQUEST['ccemail'] ) ) {
      $cc = esc_html( $_REQUEST['ccemail'] );
     } else if ( !empty( get_post_meta( $_REQUEST['postid'], 'email', true ) ) ) {
      $cc = esc_html( get_post_meta( $_REQUEST['postid'], 'email', true ) );
     }
     if ( empty( $cc ) || (!filter_var($cc, FILTER_VALIDATE_EMAIL) ) ) {
      $cc = ""; // Invalid email address
      delete_post_meta( $_REQUEST['postid'], 'email' );
     } else {
      update_post_meta( $_REQUEST['postid'], 'email', $cc );
     }
     if ( !empty( $cc ) ) {
      $to .= ',' . $cc;
     }
     //echo '<pre>' . print_r( get_option('djv_engarde_options') ) . '</pre>';
     $subject = 'En Garde order for ' . esc_html( stripslashes( $_REQUEST['characterName'] ) ) . ' for ' . esc_html( stripslashes( $_REQUEST['month'] ) );
     $headers = array('Content-Type: text/html; charset=UTF-8','From: tremlas <engarde@tremlas.com');
     if ( wp_mail( $to, $subject, $text, $headers ) ) {
      update_post_meta( $_REQUEST['postid'], 'last_orders', $text );
      if ( empty( $cc ) ) {
       echo '<p>The following En Garde order has been submitted:</p>';
      } else {
       echo '<p>The following En Garde order has been submitted and copied to ' . $cc . ':</p>';
      }
     } else {
      echo '<p><b>An error occurred sending your En Garde order to ' . $to . '.</b> The orders were: ';
     }
     echo $text;
    } else {
     echo '<p>Something went wrong with your En Garde order submission. The technical details received were:</p>';
     echo '<p><pre>' . print_r( $_REQUEST ) . '</pre></p>';
    }
  } else {
   echo '<p>Your passcode did not match that recorded for the character for which you are trying to submit orders.</p>';
  }
  if ( isset( $_REQUEST['_wp_http_referer'] ) ) {
   echo '<p>Return to <a href="' . $_REQUEST['_wp_http_referer'] .'">';
   if ( isset( $_REQUEST['characterName'] ) ) {
    echo esc_html( stripslashes( $_REQUEST['characterName'] ) ) . '\'s ';
   } 
   echo 'character sheet</a>.</p>';
  }
 }
 die();
 //request handlers should die() when they complete their task
}
add_action( 'admin_post_nopriv_djv_engarde_submit_orders', 'djv_engarde_submit_orders' );
add_action( 'admin_post_djv_engarde_submit_orders', 'djv_engarde_submit_orders' );

// show wp_mail() errors
//function djv_engarde_onMailError( $wp_error ) {
// echo "<pre>";
// print_r($wp_error);
// echo "</pre>";
//}add_action( 'wp_mail_failed', 'djv_engarde_onMailError', 10, 1 );
?>