/*!
 * © 2017 David Vines
 *
 * Creative Commons License
 *
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 */
/* globals document, console, alert */

// Polyfill for IE8 (just in case)
// Source: Eli Grey @ http://eligrey.com/blog/post/textcontent-in-ie8
if (Object.defineProperty && 
    Object.getOwnPropertyDescriptor &&
    Object.getOwnPropertyDescriptor(Element.prototype, "textContent") &&
    !Object.getOwnPropertyDescriptor(Element.prototype, "textContent").get) {
  (function() {
    var innerText = Object.getOwnPropertyDescriptor(Element.prototype, "innerText");
    Object.defineProperty(Element.prototype, "textContent",
     // Passing innerText or innerText.get directly does not work,
     // wrapper function is required.
     {
       get: function() {
         return innerText.get.call(this);
       },
       set: function(s) {
         return innerText.set.call(this, s);
       }
     }
   );
  })();
}

var returnFalse = function(f) {
    return function() {
        try {
            f();
        } catch (e) {
            if (e.stack) {
                alert(e + "\n\nStack Trace\n-----------------\n" + e.stack);
            } else {
                alert(e);
            }
        }
        return false;
    };
};

var combine = function(f,g) {
    if (!f) {
        return g;
    } else if (!g) {
        return f;
    } else {
        return function() {
            var result = f();
            if (!g()) { 
                result = false; 
            }
            return result;
        };
    }
};


function expandCollapseSection(triangle,div) {
    var obj = document.getElementById(triangle);
    if(obj.className.indexOf('arrow-down')!==-1) {
        obj.className='arrow-right';
        document.getElementById(div).style.display='none';
    }
    else {
        obj.className='arrow-down';
        document.getElementById(div).style.display='';
    }
}

function clubhouselimit(club) {
    switch(club) {
        case "Bothwell's"       : return 100000;
        case "Hunter's"         : return 200;
        case "The Horse Guards" : return 250;
        case "The Blue Gables"  : return 150;
        case "The Frog & Peach" : return 100;
        case "Red Phillips"     : return 50;
        default: return 1;
    }
}

function clubhousedivisor(club) {
    switch(club) {
        case "Bothwell's"       : return 500;
        case "Hunter's"         : return 300;
        case "The Horse Guards" : return 300;
        case "The Blue Gables"  : return 200;
        case "The Frog & Peach" : return 150;
        case "Red Phillips"     : return 150;
        default: return 1;
    }
}

function setChoiceOnChange(sl,inParis,cid,wk,club,companion,cnotes,regiment,rapier,sabre,cutlass) {
    var prefix = cid+"-week-"+wk;
    var choice = document.getElementById(prefix);
    var clubdetailscarouse = document.getElementById(prefix+"-club-details-carouse");
    var clubdetailscompanion = document.getElementById(prefix+"-club-details-companion");
    var clubdetailsgamble = document.getElementById(prefix+"-club-details-gamble");
    var bawdydetailscompanion = document.getElementById(prefix+"-bawdy-details-companion");
    var bawdydetailsgamble = document.getElementById(prefix+"-bawdy-details-gamble");
    var gambledetailsrounds = document.getElementById(prefix+"-gamble-details-rounds");
    var gambledetailsroundslabel = document.getElementById(prefix+"-gamble-details-rounds-label");
    var gambledetailsbet = document.getElementById(prefix+"-gamble-details-bet");
    var gambledetailsbetlabel = document.getElementById(prefix+"-gamble-details-bet-label");
    var gambledetailscut = document.getElementById(prefix+"-gamble-details-cut");
    var courtcompanion = document.getElementById(prefix+"-court-companion");
    var practicerapier = document.getElementById(prefix+"-practice-rapier");
    var practicesabre = document.getElementById(prefix+"-practice-sabre");
    var practicecutlass = document.getElementById(prefix+"-practice-cutlass");
    var gambleorders = function(){
        var plural = function(v,text,labeltext,label) {
            if (v === 1) {
                label.innerHTML = labeltext;
                return v+" "+text;
            } else {
                label.innerHTML = labeltext+"s";
                return v+" "+text+"s";
            }
        };
        var order = plural(gambledetailsrounds.value,"round","Round",gambledetailsroundslabel);
        order += " betting "+plural(gambledetailsbet.value,"crown","Crown",gambledetailsbetlabel);
        if (gambledetailscut.value === "7") {
            order += ", never cutting.";
        } else if (gambledetailscut.value === "1") {
            order += ", always cutting.";
        } else {
            order += ". cutting on a house roll of "+gambledetailscut.value+".";
        }
        return order;
    };
    var gambleeffects = function(club){
        var rounds = Number(gambledetailsrounds.value);
        var bet = Number(gambledetailsbet.value);
        var cut = Number(gambledetailscut.value);
        var losses = Math.round([18, 18, 11, 6, 3, 2, 3, 6,][cut]/36*rounds*bet);
        var plural = function(v,word) {
            if (v === 1) {
                return v+" "+word;
            } else {
                return v+" "+word+"s";
            }
        };
        if (club === "") {
            return "Expected loss of "+plural(losses,"Crown")+".";
        }
        var sp = Math.round((5+cut)/12*rounds*bet/clubhousedivisor(club));
        if (sp === 0) {
            return "Expected loss of "+plural(losses,"Crown")+".";
        } else if (sp === 1) {
            return "Expected loss of "+plural(losses,"Crown")+" and gain one status point.";
        } else {
            return "Expected loss of "+plural(losses,"Crown")+" and gain "+sp+" status points.";
        }
    };
    var choicefunc = returnFalse(function() {
        var chosen = choice.options[choice.selectedIndex].text;
        var oo = document.getElementById(cid+"-form-week"+wk);
        var on = document.getElementById(cid+"-form-week"+wk+"-notes");
        var oe = document.getElementById(cid+"-form-week"+wk+"-effect");
        var clubjoin = document.getElementById(cid+"-club-join");
        var clubchoice = document.getElementById(cid+"-club");
        var actualclub = club;
        if (clubjoin.checked  && clubchoice.selectedIndex > 0) {
            actualclub = clubchoice.options[clubchoice.selectedIndex].text;
        }
        if (chosen === "Visit Club" && inParis.checked) {
            document.getElementById(prefix+"-club-details").style = "";
            if (clubdetailsgamble.checked) {
                document.getElementById(prefix+"-gamble-details").style = "";
                gambledetailsbet.max = clubhouselimit(actualclub);
            } else {
                document.getElementById(prefix+"-gamble-details").style = "display: none;";
            }
        } else {
            document.getElementById(prefix+"-club-details").style = "display: none;";
        }
        if (chosen === "Visit Bawdyhouse" && inParis.checked) {
            document.getElementById(prefix+"-bawdy-details").style = "";
            if (bawdydetailsgamble.checked) {
                document.getElementById(prefix+"-gamble-details").style = "";
                gambledetailsrounds.max = 10000;
            } else {
                document.getElementById(prefix+"-gamble-details").style = "display: none;";
            }
        } else {
            document.getElementById(prefix+"-bawdy-details").style = "display: none;";
        }
        if ( (chosen !== "Visit Club") && (chosen !== "Visit Bawdyhouse") ) {
            document.getElementById(prefix+"-gamble-details").style = "display: none;";
        }
        if (chosen === "Practice Fencing" && inParis.checked) {
            document.getElementById(prefix+"-practice-details").style = "";
        } else {
            document.getElementById(prefix+"-practice-details").style = "display: none;";
        }
        if (chosen === "Court new female companion" && inParis.checked) {
            document.getElementById(prefix+"-court-details").style = "";
        } else {
            document.getElementById(prefix+"-court-details").style = "display: none;";
        }
        if (!inParis.checked) {
            document.getElementById(prefix+"-effect").innerHTML = "<p>Out of Paris</p>";
            oo.value = "";
            on.value = "";
            oe.value = "Out of Paris";
        } else {
            var orders = "";
            var effect = "";
            var cost = 0;
            var needgamblelink = false;
            if (chosen === "Visit Club") {
                orders = "Visit "+actualclub;
                if (clubdetailscarouse.checked) {
                    orders += ", carouse";
                    effect = " Gain one status point";
                    cost = Number(sl);
                }
                if (clubdetailscompanion.checked) {
                    orders += " with companion";
                    if (cnotes) {
                        cost += Number(cnotes.match(/\d+/)[0]);
                    }
                    effect = "Estimated cost of "+cost+" Crowns." + effect;
                 } else {
                    effect = "Costing "+cost+" Crowns." + effect;
                }
                if (clubdetailsgamble.checked) {
                    orders += " and gamble "+gambleorders();
                    effect += " "+gambleeffects(actualclub);
                    needgamblelink = true;
                }
            } else if (chosen === "Visit Bawdyhouse") {
                orders = "Visit Bawdyhouse, carouse";
                cost = Number(sl);
                if (bawdydetailscompanion.checked) {
                    orders += " with bawdyhouse companion";
                    cost += Number(sl);
                }
                effect = "Costing "+cost+" Crowns. Gain one status point. There is a 1 in 6 chance of being robbed in the red light district. You may wish to specify how much cash you're carrying when you enter the bawdyhouse (If not specified, it will be assumed that you're bringing 25% of your cash with you).";
                if (bawdydetailsgamble.checked) {
                    orders += " and gamble "+gambleorders();
                    effect += " "+gambleeffects("");
                    needgamblelink = true;
                }
            }  else if (chosen === "Court new female companion") {
                var compchosen = courtcompanion.options[courtcompanion.selectedIndex].text;
                if (compchosen) {
                    orders = "Court "+compchosen;
                    var hersl = compchosen.match(/\d+/)[0];
                    var hername = compchosen.match(/^(.*) \(/)[1];
                    cost = hersl*3;
                    var diff = sl-hersl;
                    var odds = 2;
                    if (diff < 3)   { odds = 3; }
                    if (diff < 0)   { odds = 4; }
                    if (diff < -3)  { odds = 5; }
                    if (diff ===-6) { odds = 6; }
                    if (diff < -6) {
                        effect = "Courting "+hername+" is impossible for one with your low social level";
                    } else {
                        effect = "Courting "+hername+" costs a minimum of "+cost+" Crowns. You need a "+odds+" or better on 1d6 to succeed. Modifiers: -1 if she already has a partner";
                        if (compchosen.indexOf("Wealthy") === -1) {
                            effect += ", +1 for each extra expenditure of "+cost+" Crowns";
                        }
                        effect += ". A roll of 1 always fails.";
                    }
                }
            } else if (chosen === "On Duty") {
                orders = "On Duty";
                effect = "";
            } else if (chosen === "Visit female companion") {
                orders = "Visit "+companion;
                effect = "No additional expense.";
            } else if (chosen === "Practice Fencing") {
                var rw = regWeapon(regiment);
                if (practicerapier.checked) {
                    orders = "Practice Rapier";
                    if ( rw !== "Rapier" ) {
                        effect = "Your Practice costs "+rapier+" Crowns.";
                    } else {
                        effect = "Your regiment will train you for free.";
                    }
                }
                if (practicesabre.checked) {
                    orders = "Practice Sabre";
                    if ( rw !== "Sabre" ) {
                        effect = "Your Practice costs "+rapier+" Crowns.";
                    } else {
                        effect = "Your regiment will train you for free.";
                    }
                }
                if (practicecutlass.checked) {
                    orders = "Practice Cutlass";
                    if ( rw !== "Cutlass" ) {
                        effect = "Your Practice costs "+rapier+" Crowns.";
                    } else {
                        effect = "Your regiment will train you for free.";
                    }
                }
            }
            oo.value = orders;
            on.value = document.getElementById(prefix+"-notes").value;
            if (needgamblelink) {
                document.getElementById(prefix+"-effect").innerHTML = '<p>'+effect+' (<a href="/games/en-garde/gambling/">Gambling Rules</a>)';
            } else {
                document.getElementById(prefix+"-effect").innerHTML = '<p>'+effect+"</p>";
            }
            oe.value = effect;
        } 
    });
    // Fire to get form display to match the current data
    choicefunc();
    choice.onchange = combine(choicefunc,choice.onchange);
    clubdetailscarouse.onchange = combine(choicefunc,clubdetailscarouse.onchange);
    clubdetailscompanion.onchange = combine(choicefunc,clubdetailscompanion.onchange);
    clubdetailsgamble.onchange = combine(choicefunc,clubdetailsgamble.onchange);
    bawdydetailscompanion.onchange = combine(choicefunc,bawdydetailscompanion.onchange);
    bawdydetailsgamble.onchange = combine(choicefunc,bawdydetailsgamble.onchange);
    gambledetailsrounds.onchange = combine(choicefunc,gambledetailsrounds.onchange);
    gambledetailsbet.onchange = combine(choicefunc,gambledetailsbet.onchange);
    gambledetailscut.onchange = combine(choicefunc,gambledetailscut.onchange);
    courtcompanion.onchange = combine(choicefunc,courtcompanion.onchange);
    practicerapier.onchange = combine(choicefunc,practicerapier.onchange);
    practicesabre.onchange = combine(choicefunc,practicesabre.onchange);
    practicecutlass.onchange = combine(choicefunc,practicecutlass.onchange);
    document.getElementById(prefix+"-notes").onchange = combine(choicefunc, document.getElementById(prefix+"-notes").onchange);
}

function stableCost(stable) {
    if (stable) {
        var horses = stable.match(/\d+/)[0];
        return horses*3+2;
    } else {
        return 0;
    }
}

function setSupportNone(allowance,inParis,stable,cid) {
    var button = document.getElementById(cid+"-support-none");
    var buttonfunc = returnFalse(function() {
        if (button.checked) {
            var notes = document.getElementById(cid+"-support-effect");
            var form = document.getElementById(cid+"-form-support");
            var oe = document.getElementById(cid+"-form-support-effect");
            var effect;
            if (inParis.checked) {
                effect = "0 Crowns spent of an income of "+allowance+" Crowns. Social Level will decrease by one.";
            } else {
                effect = "Out of Paris - no support costs - all your income of "+allowance+" Crowns will be saved.";
            }
            if (stable) {
                effect += " Your stable of "+stable+" will cost an additional "+stableCost(stable)+" Crowns.";
            }
            notes.innerHTML = "<p>"+effect+"</p>";
            form.value = "None";
            oe.value = effect;
        }
    });
    buttonfunc(); // Initialise notes text if applicable
    button.onchange = combine(buttonfunc,button.onchange);
}

function stableExpense(stable) {
    if (stable) {
        return " Your stable of "+stable+" will cost an additional "+stableCost(stable)+" Crowns.";
    } else {
        return "";
    }
}

function mistressExpense(sl,companion,cnotes) {
    if (companion) {
        var hersl = cnotes.match(/\d+/)[0];
        if (cnotes.indexOf("Wealthy") === -1) {
            return " Supporting "+companion+" will cost an additional "+(hersl*3)+" Crownsto keep her in the manner to which she is accustomed.";
        } else {
            if (hersl > sl) {
                return " "+companion+" makes small gifts to you worth a total of "+(hersl-sl)*2+" Crowns.";
            } else {
                return " "+companion+"'s wealth is able to keep her in the manner to which she is accustomed.";
            }
        }
    } else {
        return "";
    }
}

function setSupportStd(sl,allowance,inParis,stable,companion,cnotes,cid) {
   var button = document.getElementById(cid+"-support-std");
   var buttonfunc = returnFalse(function() {
        if (button.checked) {
            var notes = document.getElementById(cid+"-support-effect");
            var form = document.getElementById(cid+"-form-support");
            var oe = document.getElementById(cid+"-form-support-effect");
            var effect;
            if (inParis.checked) {
                effect = ""+sl*2+" Crowns spent of an income of "+allowance+" Crowns.";
            } else {
                effect = "Out of Paris - no support costs - all your income of "+allowance+" Crowns will be saved.";
            }
            effect += stableExpense(stable);
            effect += mistressExpense(sl,companion,cnotes);
            notes.innerHTML = "<p>"+effect+"</p>";
            form.value = "Standard";
            oe.value = effect;
        }
    });
   buttonfunc(); // Initialise notes text if applicable
   button.onchange = combine(buttonfunc,button.onchange);
   inParis.onchange = combine(buttonfunc,inParis.onchange);
}

function setSupportCon(sl,allowance,inParis,stable,companion,cnotes,cid) {
    var button = document.getElementById(cid+"-support-con");
    var buttonfunc = returnFalse(function() {
        var oe = document.getElementById(cid+"-form-support-effect");
        if (button.checked) {
            var notes = document.getElementById(cid+"-support-effect");
            var form = document.getElementById(cid+"-form-support");
            var effect;
            if (inParis.checked) {
                effect = ""+sl*3+" Crowns spent of an income of "+allowance+" Crowns. 1 Status Point gained.";
            } else {
                effect = "Out of Paris - no support cost and no need/effect for conspicious consumption  - all your income of "+allowance+" Crowns will be saved.";
            }
            effect += stableExpense(stable);
            effect += mistressExpense(sl,companion,cnotes);
            notes.innerHTML = "<p>"+effect+"</p>";
            form.value = "Conspicious";
            oe.value = effect;
        }
    });
    buttonfunc(); // Initialise notes text if applicable
    button.onchange = combine(buttonfunc,button.onchange);
    inParis.onchange = combine(buttonfunc,inParis.onchange);
}

function getSocialLevelOfClub(club) {
    switch(club) {
        case "Bothwell's"       : return 12;
        case "Hunter's"         : return 9;
        case "The Blue Gables"  : return 7;
        case "The Frog & Peach" : return 5;
        case "Red Phillips"     : return 3;
        default: return 0;
    }
}

function setClub(sl,inParis,regiment,club,cid) {
    var leave = document.getElementById(cid+"-club-leave");
    var join = document.getElementById(cid+"-club-join");
    var choice = document.getElementById(cid+"-club");
    var clubfunc = returnFalse(function() {
        var notes = document.getElementById(cid+"-club-effect");
        var form = document.getElementById(cid+"-form-club");
        var oe = document.getElementById(cid+"-form-club-effect");
        var newclub = (leave.checked ? "" : (club ? club.trim() : ""));
        var order = "No Change";
        var text = ""; // Default text
        if (leave.checked) {
            order = "Leave "+(club ? club.trim() : "")+". ";
        }
        if (join.checked  && choice.selectedIndex > 0) {
            var newch = choice.options[choice.selectedIndex].text;
            order += "Join "+newch+".";
            if (newch === "The Horse Guards") {
                if (regiment && (regiment.startsWith("Dragoon Guards") || regiment.startsWith("Queen's Own Carabiniers"))) {
                    newclub = "The Horse Guards";
                    text = "Joined "+newclub+". ";
                } else {
                    text = "Application refused unless you succeed in an application to join the Dragoon Guards or the Queen's Own Carabiniers. ";
                }
            } else if (sl >= getSocialLevelOfClub(newch)) {
                newclub = newch;
                text = "Joined "+newclub+". ";
            } else if (newch !== "") {
                text = "Application to "+newch+" rejected. "+newch+" requires applicants to have a minimum social level of "+getSocialLevelOfClub(newch)+".";
            }
        }
        if (inParis.checked) {
            switch(newclub) {
                case "The Horse Guards" :
                    text += "Dues of 20 Crowns. Gain 4 Status Points.";
                    break;
                case "Bothwell's": 
                    text += "Dues of 30 Crowns. Gain 8 Status Points.";
                    break;
                case "Hunter's": 
                    text += "Dues of 20 Crowns. Gain 6 Status Points.";
                    break;
                case "The Blue Gables": 
                    text += "Dues of 15 Crowns. Gain 4 Status Points.";
                    break;
                case "The Frog & Peach":
                    text += "Dues of 10 Crowns. Gain 3 Status Points.";
                    break;
                case "Red Phillips":
                    text += "Dues of 5 Crowns. Gain 2 Status Points.";
                    break;
                default:
                    break;
            }
        } else {
            text += "Cannot change club while out of Paris (no club fees will be levied however).";
        }
        notes.innerHTML = "<p>"+text+"</p>";
        form.value = order;
        oe.value = text;
    });
    // Disable no-effect options
    if (!club) {
        document.getElementById(cid+"-club-leave").disabled = true;
        document.getElementById(cid+"-club-leave-label").style="opacity: 0.5;";
    } else {
        document.getElementById(cid+"-club-leave").disabled = false;
        document.getElementById(cid+"-club-leave-label").style="";
    }
    clubfunc();  // Initialise notes text if applicable
    leave.onchange = combine(clubfunc,leave.onchange);
    join.onchange = combine(clubfunc,join.onchange);
    choice.onchange = combine(clubfunc,choice.onchange);
    inParis.onchange = combine(clubfunc,inParis.onchange);

}

function regimentColumn(regiment) {
    switch (regiment.split(',')[0].trim()) {
        case "Dragoon Guards"                   : return 3;
        case "Queen's Own Carabiniers"          : return 4;
        case "Royal Foot Guards"                : return 1;
        case "King's Musketeers"                : return 2;
        case "Cardinal's Guard"                 : return 2;
        case "Crown Prince Cuirassiers"         : return 5;
        case "Archduke Leopold Cuirassiers"     : return 4;
        case "Grand Duke Maximillian Dragoons"  : return 6;
        case "Princess Louisa Light Dragoons"   : return 8;
        case "Royal Marines"                    : return 6;
        case "Picardy Musketeers"               : return 7; 
        case "13th Fusiliers"                   : return 8;
        case "53rd Fusiliers"                   : return 9;
        case "27th Musketeers"                  : return 9;
        case "4th Arquebusiers"                 : return 9;
        case "69th Arquebusiers"                : return 10;
        case "Gascon Regiment"                  : return 10;
        default: return 10;
    }
}

function isCavalry(regiment) {
    switch (regiment.split(',')[0].trim()) {
        case "Dragoon Guards"                   : return true;
        case "Queen's Own Carabiniers"          : return true;
        case "Royal Foot Guards"                : return false;
        case "King's Musketeers"                : return false;
        case "Cardinal's Guard"                 : return false;
        case "Crown Prince Cuirassiers"         : return true;
        case "Archduke Leopold Cuirassiers"     : return true;
        case "Grand Duke Maximillian Dragoons"  : return true;
        case "Princess Louisa Light Dragoons"   : return true;
        case "Royal Marines"                    : return false;
        case "Picardy Musketeers"               : return false; 
        case "13th Fusiliers"                   : return false;
        case "53rd Fusiliers"                   : return false;
        case "27th Musketeers"                  : return false;
        case "4th Arquebusiers"                 : return false;
        case "69th Arquebusiers"                : return false;
        case "Gascon Regiment"                  : return false;
        default: return false;
    }
}

function regWeapon(regiment) {
    if (!regiment) {
        return "";
    }
    switch (regiment.split(',')[0].trim()) {
        case "Dragoon Guards"                   : return "Sabre";
        case "Queen's Own Carabiniers"          : return "Sabre";
        case "Crown Prince Cuirassiers"         : return "Sabre";
        case "Archduke Leopold Cuirassiers"     : return "Sabre";
        case "Grand Duke Maximillian Dragoons"  : return "Sabre";
        case "Princess Louisa Light Dragoons"   : return "Sabre";
        case "Royal Marines"                    : return "Cutlass";
        default: return "Rapier";
    }
}

function regbasesl(regiment) {
    var reg = regiment.split(',')[0];
    var col = regimentColumn(reg);
    return [7,6,5,4,3,2,0,0,0,0][col];
}

function regsalary(regiment) {
    var reg = regiment.split(',')[0].trim();
    var rank = regiment.split(',')[1].trim();
    var col = regimentColumn(reg);
    switch(rank) {
        case "Private":   return [12,10,10,8,8,6,4,4,2,2][col-1];
        case "Subaltern": return [16,14,14,10,10,8,6,6,4,4][col-1];
        case "Captain":   return [18,16,16,14,14,12,10,10,8,8][col-1];
        case "Major":     return [22,20,20,18,18,16,14,14,12,12][col-1];
        case "Lieutenant Colonel": return [26,24,24,22,22,20,18,18,16,16][col-1];
        case "Colonel": return [30,28,28,26,26,24,22,22,20,20][col-1];
        default: return 0;
    }
}

function regsp(regiment) {
    var reg = regiment.split(',')[0].trim();
    var rank = regiment.split(',')[1].trim();
    var col = regimentColumn(reg);
    switch(rank) {
        case "Private":   return [6,5,4,3,3,2,1,0,0,0][col-1];
        case "Subaltern": return [7,6,5,4,4,3,2,1,0,0][col-1];
        case "Captain":   return [8,7,6,5,5,4,3,2,1,0][col-1];
        case "Major":     return [9,8,7,6,6,5,4,3,2,1][col-1];
        case "Lieutenant Colonel": return [10,9,8,7,7,6,5,4,3,2][col-1];
        case "Colonel": return [11,10,9,8,7,7,6,5,4,3][col-1];
        default: return 0;
    }
}

function regjoinroll(regiment,sl) {
    var roll = 9-sl; // Will be halved and rounded down at the end
    switch (regiment) { // Note
        case "Royal Foot Guards"                : roll += 9; break;
        case "Cardinal's Guard"                 : roll += 8; break;
        case "King's Musketeers"                : roll += 8; break;
        case "Dragoon Guards"                   : roll += 7; break;
        case "Queen's Own Carabiniers"          : roll += 6; break;
        case "Archduke Leopold Cuirassiers"     : roll += 6; break;
        case "Crown Prince Cuirassiers"         : roll += 5; break;
        case "Royal Marines"                    : roll += 4; break;
        case "Grand Duke Maximillian Dragoons"  : roll += 4; break;
        case "Picardy Musketeers"               : roll += 3; break;
        case "13th Fusiliers"                   : roll += 3; break;
        case "Princess Louisa Light Dragoons"   : roll += 2; break;
        case "53rd Fusiliers"                   : roll += 2; break;
        case "27th Musketeers"                  : roll += 2; break;
        case "4th Arquebusiers"                 : roll += 1; break;
        case "69th Arquebusiers"                : roll += 1; break;
        case "Gascon Regiment"                  : break;
        default: break;
    }
    return Math.floor(roll/2);
}

function setRegiment(sl,inParis,regiment,stable,cid) {
    var leave = document.getElementById(cid+"-regiment-leave");
    var join = document.getElementById(cid+"-regiment-join");
    var choice = document.getElementById(cid+"-regiment");
    var regfunc = returnFalse(function() {
        var notes = document.getElementById(cid+"-regiment-effect");
        var form = document.getElementById(cid+"-form-regiment");
        var oe = document.getElementById(cid+"-form-regiment-effect");
        var newreg = (leave.checked ? "" : (regiment ? regiment.trim() : ""));
        var order = "No Change";
        var text = ""; // Default text
        var failed = false;
        if (leave.checked) {
            order = "Leave "+(regiment ? regiment.trim() : "")+". ";
            text = "Salary of "+regsalary(regiment)+" Crowns lost. ";
        }
        if (join.checked && choice.selectedIndex > 0) {
            var newch = choice.options[choice.selectedIndex].text;
            order += "Join "+newch+".";
            if (sl >= regbasesl(newch)) {
                newreg = newch+", Private";
              text += "Need a roll of "+regjoinroll(newch,sl)+" or more on 1d6 for application to succeed. ";
              if (isCavalry(newch)) {
                  text += "As "+newch+" is a cavalry regiment you will need to ensure that you own at least one horse if your application is successful. ";
              }
              text += "If successful you have a salary of "+regsalary(newreg)+" as a private";
              if (inParis.checked) {
                  text += " and ";
              } else {
                  text += ".";
              }
            } else {
              text += newch+" requires its applicants to have a minimum social level of "+regbasesl(newch)+" so your application will fail.";
              failed = false;
            }
        }
        if (newreg && inParis.checked && !failed) {
            if (text === "") {
                text = "Gain";
            } else {
                text += "gain";
            }
            text += " "+regsp(newreg)+" status points this month.";
        }
        if (!inParis.checked) {
            text += "Cannot change regiment while out of Paris (you still draw your salary however)";
        }
        notes.innerHTML = "<p>"+text+"</p>";
        form.value = order;
        oe.value = text;
    });
    regfunc();  // Initialise notes text if applicable
    if (!regiment || !inParis.checked) {
        document.getElementById(cid+"-regiment-leave").disabled = true;
        document.getElementById(cid+"-regiment-leave-label").style="opacity: 0.5;";
    } else {
        document.getElementById(cid+"-regiment-leave").disabled = false;
        document.getElementById(cid+"-regiment-leave-label").style="";
    }
    
    leave.onchange = combine(regfunc,leave.onchange);
    join.onchange = combine(regfunc,join.onchange);
    choice.onchange = combine(regfunc,choice.onchange);
    inParis.onchange = combine(regfunc,inParis.onchange);
}

function setPromote(sl,inParis,regiment,stable,cid) {
    var leave = document.getElementById(cid+"-regiment-leave");
    var join = document.getElementById(cid+"-regiment-join");
    var newreg = document.getElementById(cid+"-regiment");
    var promote = document.getElementById(cid+"-promote");
    var choice = document.getElementById(cid+"-rank");
    var profunc = returnFalse(function() {
        var notes = document.getElementById(cid+"-rank-effect");
        var form = document.getElementById(cid+"-form-rank");
        var oe = document.getElementById(cid+"-form-rank-effect");
        if (inParis) {
            if (promote.checked  && (regiment || (join.checked && newreg.selectedIndex > 0)) && choice.selectedIndex > 0) {
                var newrank = choice.options[choice.selectedIndex].text;
                var actual = regiment;
                var text = "Assuming";
                var order = "Apply for "+newrank+".";
                if (join.checked && newreg.selectedIndex > 0) {
                    var newRegiment = newreg.options[newreg.selectedIndex].text;
                    actual = newRegiment+", Private";
                    text = "Assuming your application to "+newRegiment+" is successful and assuming";
                }
                text += " there's is a position available, your promotion to "+newrank+" will cost ";
                var reg = actual.split(',')[0].trim();
                var oldrank = actual.split(',')[1].trim();
                var col = regimentColumn(reg);
                var cost = 0;
                if (oldrank === "Private") {
                    cost += 160 - 10*col;
                    oldrank = "Subaltern";
                }
                if (oldrank === "Subaltern" && (newrank === "Captain" || newrank === "Major" || newrank === "Lieutenant Colonel" || newrank === "Colonel")) {
                    cost += 170 - 10*col;
                    oldrank = "Captain";
                }
                if (oldrank === "Captain" && (newrank === "Major" || newrank === "Lieutenant Colonel" || newrank === "Colonel")) {
                    cost += 180 - 10*col;
                    oldrank = "Major";
                }
                if (oldrank === "Major" && (newrank === "Lieutenant Colonel" || newrank === "Colonel")) {
                    cost += 200 - 10*col;
                    oldrank = "Lieutenant Colonel";
                }
                if (oldrank === "Lieutenant Colonel" && newrank === "Colonel") {
                    cost += 250 - 10*col;
                    oldrank = "Colonel";
                }
                text += cost+" Crowns.";
                if (newrank === "Captain" && !stable) {
                    text += " As a captain you will also need a horse (each horse costs 100 Crowns and then 3 Crowns per month. If you have any horses you also need to hire a (single) groom at a cost of 2 Crowns per month).";
                }
                if (newrank === "Major" || newrank === "Lieutenant Colonel" || newrank === "Colonel") {
                    var horses = (stable ? stable.match(/\d+/)[0] : 0);
                    if (horses < 3) {
                        text += " As a "+newrank+" you will also need three horses (each horse costs 100 Crowns and then 3 Crowns per month. If you have any horses you also need to hire a (single) groom at a cost of 2 Crowns per month).";
                    }
                }
                text += " Your salary will increase to "+regsalary(reg+", "+newrank)+" Crowns from "+regsalary(actual)+" Crowns.";
                text += " You status points from your regiment will increase to "+regsp(reg+", "+newrank)+".";
                notes.innerHTML = "<p>"+text+"</p>";
                form.value = order;
                oe.value = text;
            } else {
                notes.innerHTML = ""; // No notes if you're not promoting
                form.value = "No Change";
                oe.value = "";
            }
        } else {
            notes.innerHTML = "<p>Cannot change rank while out of Paris.</p>"; // No notes if you're not promoting
            form.value = "No Change";
            oe.value = "Cannot change rank while out of Paris.";
        }
    });
    profunc(); //Initialise notes text if applicable
    leave.onchange = combine(profunc,leave.onchange);
    join.onchange = combine(profunc,join.onchange);
    newreg.onchange = combine(profunc,newreg.onchange);
    promote.onchange = combine(profunc,promote.onchange);
    choice.onchange = combine(profunc,choice.onchange);
}

function setLocation(inParis,location,cid) {
    var notes = document.getElementById(cid+"-location-notes");
    var locfunc = returnFalse(function(){
        var atwar = document.getElementById(cid+"-location-at-war");
        var frontier = document.getElementById(cid+"-location-frontier");
        var other = document.getElementById(cid+"-location-other");
        var eff = document.getElementById(cid+"-location-effect");
        var form = document.getElementById(cid+"-form-location");
        var fln = document.getElementById(cid+"-form-location-notes");
        var fle = document.getElementById(cid+"-form-location-effect");
        var text;
        var effect = "";
        if (inParis.checked) {
            text = "In Paris";
        } else if (atwar.checked) {
            text = "Gone with Regiment";
            effect = "You must remain with your regiment until the end of the campaigning season.";
        } else if (frontier.checked) {
            text = "Gone with the Frontier Regiment";
            effect = "You must remain with your regiment until the end of the campaigning season.";
        } else if (other.checked) {
            text = "Out of Paris";
            effect = "You may return next month if desired.";
        } else {
            text = "Unknown";
        }
        
        // Set up state of the other form elements
        var disabled = (!inParis.checked);
        var opacity = (inParis.checked ? "" : "opacity: 0.5;");

        if (!inParis.checked) {
            document.getElementById(cid+"-support-none").disabled = disabled;
            document.getElementById(cid+"-support-none-label").style = opacity;
            document.getElementById(cid+"-support-std").disabled = disabled;
            document.getElementById(cid+"-support-std-label").style = opacity;
            document.getElementById(cid+"-support-con").disabled = disabled;
            document.getElementById(cid+"-support-con-label").style = opacity;

            document.getElementById(cid+"-club-leave").disabled = disabled;
            document.getElementById(cid+"-club-leave-label").style = opacity;
            document.getElementById(cid+"-club-join").disabled = disabled;
            document.getElementById(cid+"-club-join-label").style = opacity;
            document.getElementById(cid+"-club").disabled = disabled;
            document.getElementById(cid+"-club").style = opacity;

            document.getElementById(cid+"-regiment-leave").disabled = disabled;
            document.getElementById(cid+"-regiment-leave-label").style = opacity;
            document.getElementById(cid+"-regiment-join").disabled = disabled;
            document.getElementById(cid+"-regiment-join-label").style = opacity;
            document.getElementById(cid+"-regiment").disabled = disabled;
            document.getElementById(cid+"-regiment").style = opacity;

            document.getElementById(cid+"-promote").disabled = disabled;
            document.getElementById(cid+"-promote-label").style = opacity;
            document.getElementById(cid+"-rank").disabled = disabled;
            document.getElementById(cid+"-rank").style = opacity;
            for(var i=1; i<5; i++) {
                document.getElementById(cid+"-week-"+i+"-heading").style = opacity;
                document.getElementById(cid+"-week-"+i).disabled = disabled;
                document.getElementById(cid+"-week-"+i+"-notes-label").style = opacity;
                document.getElementById(cid+"-week-"+i+"-notes").disabled = disabled;
            }
        }
        
        eff.innerHTML = "<p>"+effect+"</p>";
        form.value = text;
        fln.value = notes.value;
        fle.value = effect;
    });
    // If at war or the frontier in the stats table location can't change so disable (and set buttons) before driving locfunc 
    if ( (location === 'At War') || (location === 'At Frontier') ) {
        document.getElementById(cid+"-location-paris").disabled = true;
        document.getElementById(cid+"-location-paris-label").style="opacity: 0.5;";
        document.getElementById(cid+"-location-at-war").disabled = true;
        document.getElementById(cid+"-location-at-war").checked = (location === "At War");
        document.getElementById(cid+"-location-at-war-label").style="opacity: 0.5;";
        document.getElementById(cid+"-location-frontier").disabled = true;
        document.getElementById(cid+"-location-frontier").checked = (location === "At Frontier");
        document.getElementById(cid+"-location-frontier-label").style="opacity: 0.5;";
        document.getElementById(cid+"-location-other").disabled = true;
        document.getElementById(cid+"-location-other-label").style="opacity: 0.5;";
    }
    locfunc(); // Initialise notes text if applicable
    inParis.onchange = combine(locfunc,inParis.onchange); // Other buttons will be applied by setupform
    notes.onchange = combine(locfunc,notes.onchange); 
}

function setOther(cid,inParis,regiment) {
    var other = document.getElementById(cid+"-other-notes");
    var wks = [];
    var clubcom = [];
    var bawdycom = [];
    for(var i=1; i<5; i++) {
        wks[i] = document.getElementById(cid+"-week-"+i);
        clubcom[i] = document.getElementById(cid+"-week-"+i+"-club-details-companion");
        bawdycom[i] = document.getElementById(cid+"-week-"+i+"-bawdy-details-companion");
    }
    var otherfunc = returnFalse(function(){
        var notes = document.getElementById(cid+"-other-notes");
        var effect = document.getElementById(cid+"-other-notes-effect");
        var text = "";
        if (inParis.checked) {
            var visits = 0;
            var duties = 0;
            for(i=1; i<5; i++) {
                var chosen = wks[i].options[wks[i].selectedIndex].text;
                if (chosen === "On Duty") {
                    duties++;
                }
                if (chosen === "Visit Club" && clubcom[i].checked) {
                    visits++;
                }
                if (chosen === "Visit Bawdyhouse" && bawdycom[i].checked) {
                    visits++;
                }
                if (chosen === "Court new female companion") {
                    visits += 0.1;
                }
                if (chosen === "Visit female companion") {
                    visits++;
                }
            }
            if (regiment) {
                var rank = regiment.split(',')[1].trim();
                if (duties === 1) {
                    if (rank === "Private") {
                        text += "As a private you will need permission (granted on a roll of 5 on 1d6) from their battalion commander to skip one week (of the required two weeks) of duty. ";
                    }
                } else if (duties === 0) {
                    if (rank === "Private") {
                        text += "As a private you must spend at least one week on duty. ";
                    } else if (rank === "Subaltern") {
                        text += "As a subaltern you will need permission (granted on a roll of 5 on 1d6) from their battalion commander to skip their one week of duty. ";
                    }
                }
            }
        
            if (visits === 0) {
                text += "You have not made a visit to a female companion this month. You will lose two status points (cumulative) for every month in a row that you are without a female companion.";
            } else if (visits < 1) {
                text += "If your courtship fails you are at risk of being without a female companion this month. If so you will lose two status points (cumulative) for every month in a row that you are without a female companion.";
            }
        } else {
            text = "";
        }
        
        effect.innerHTML = "<p>"+text+"</p>";
        var on = document.getElementById(cid+"-form-other-notes");
        var oe = document.getElementById(cid+"-form-other-effect");
        on.value = notes.value;
        oe.value = text;
    });
    otherfunc();
    other.onchange = combine(otherfunc, other.onchange);
    inParis.onchange = combine(otherfunc,inParis.onchange); // Other location buttons will be applied by setupform
    for(i=1; i<5; i++) {
        wks[i].onchange = combine(otherfunc,wks[i].onchange);
        clubcom[i].onchange = combine(otherfunc,clubcom[i].onchange);
        bawdycom[i].onchange = combine(otherfunc,bawdycom[i].onchange);
    }
}

function setupform(table, cid) {
    var contents = [];
    var notes = [];
    contents.Location = 'In Paris'; // default the location entry
    for(var row=table.firstElementChild.firstElementChild; row!==null; row=row.nextElementSibling) {
        contents[row.firstElementChild.textContent.trim()] = row.firstElementChild.nextElementSibling.textContent.trim();
        notes[row.firstElementChild.textContent.trim()] = row.firstElementChild.nextElementSibling.nextElementSibling.textContent.trim();
    }
    
    var sl = contents['Social Level'];
    var regiment = contents['Regiment & Rank'];
    var allowance = contents['Allowance, Salary & Pension'];
    var stable = contents.Stable;
    var inParis = document.getElementById(cid+"-location-paris");
    
    // set up location first (it may change the button settings and disable quite a few
    setLocation(inParis,contents.Location,cid);
    
    // Set up listeners (and get the form initialised via those listeners)
    setSupportNone(allowance,inParis,stable,cid);
    setSupportStd(sl,allowance,inParis,stable,contents['Female Companion'],notes['Female Companion'],cid);
    setSupportCon(sl,allowance,inParis,stable,contents['Female Companion'],notes['Female Companion'],cid);
    setClub(sl,inParis,regiment,contents.Club,cid);
    setRegiment(sl,inParis,regiment,stable,cid);
    setPromote(sl,inParis,regiment,stable,cid);
    setOther(cid,inParis,regiment);
    
    // Setup weekly rows
    for(var wk=1; wk<=4; wk++) {
        setChoiceOnChange(sl,inParis,cid,wk,contents.Club,contents['Female Companion'],notes['Female Companion'],regiment,contents['Rapier Expertise'],contents['Sabre Expertise'],contents['Cutlass Expertise']);
      }
    // Copy the handlers from paris to the other locations
    document.getElementById(cid+"-location-at-war").onchange = inParis.onchange;
    document.getElementById(cid+"-location-frontier").onchange = inParis.onchange;
    document.getElementById(cid+"-location-other").onchange = inParis.onchange;
}

var djvengardeTables = document.querySelectorAll('[id^="djv-engarde-stats-"]');
for(var i=0; i<djvengardeTables.length; i++) {
    var characterId = djvengardeTables[i].id.split('-').pop();
    var form = document.getElementById('djv-engarde-'+characterId+'-order-form');
    if (form) {
        try {
            setupform(djvengardeTables[i],'djv-engarde-'+characterId);
        } catch (e) {
            if (e.stack) {
                alert(e + "\n\nStack Trace\n-----------------\n" + e.stack);
            } else {
                alert(e);
            }
        }
    } else {
        console.log("Only found "+djvengardeTables[i].id+" (characterId="+characterId+")");
    }
}